import pytest
import requests
import responses
from pytest_mock import MockerFixture

from anaconda_cloud_auth.config import AuthConfig


@pytest.fixture(autouse=True)
def mock_openid_configuration():
    """Mock return value of openid configuration to prevent requiring actual network calls."""
    expected = {
        "authorization_endpoint": "https://anaconda.cloud/authorize",
        "token_endpoint": "https://anaconda.cloud/api/iam/token",
        "jwks_uri": "NOT_NEEDED_FOR_TESTS",
    }
    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        # These reflect the two URLs, for which there is a CloudFlare rule to return the same
        # configuration.
        # TODO: Adjust once we are confident we don't need that CloudFlare rule anymore
        rsps.get(
            url="https://id.anaconda.cloud/.well-known/openid-configuration",
            json=expected,
        )
        rsps.get(
            url="https://anaconda.cloud/api/iam/.well-known/openid-configuration",
            json=expected,
        )
        yield rsps


def test_legacy() -> None:
    config = AuthConfig(domain="anaconda.cloud/api/iam")
    assert config.oidc.authorization_endpoint == "https://anaconda.cloud/authorize"
    assert config.oidc.token_endpoint == "https://anaconda.cloud/api/iam/token"


def test_well_known_headers(mocker: MockerFixture) -> None:
    spy = mocker.spy(requests, "get")

    config = AuthConfig()
    assert config.oidc
    spy.assert_called_once()
    assert (
        spy.call_args.kwargs.get("headers", {})
        .get("User-Agent")
        .startswith("anaconda-cloud-auth")
    )



import typer
from rich.prompt import Confirm

from anaconda_cli_base.console import console
from anaconda_cloud_auth import __version__
from anaconda_cloud_auth.actions import is_logged_in
from anaconda_cloud_auth.actions import login
from anaconda_cloud_auth.actions import logout
from anaconda_cloud_auth.client import BaseClient
from anaconda_cloud_auth.config import AuthConfig
from anaconda_cloud_auth.config import APIConfig
from anaconda_cloud_auth.token import TokenInfo
from anaconda_cloud_auth.token import TokenNotFoundError

app = typer.Typer(
    name="cloud", add_completion=False, help="Anaconda.cloud auth commands"
)


@app.callback(invoke_without_command=True)
def main(version: bool = typer.Option(False, "-V", "--version")) -> None:
    if version:
        console.print(
            f"Anaconda Cloud Auth, version [cyan]{__version__}[/cyan]",
            style="bold green",
        )
        raise typer.Exit()


@app.command("login")
def auth_login(force: bool = False, ssl_verify: bool = True) -> None:
    """Login"""
    try:
        auth_domain = AuthConfig().domain
        expired = TokenInfo.load(domain=auth_domain).expired
        if expired:
            console.print("Your API key has expired, logging into Anaconda.cloud")
            login(force=True, ssl_verify=ssl_verify)
            raise typer.Exit()
    except TokenNotFoundError:
        pass  # Proceed to login
    else:
        force = force or Confirm.ask(
            f"You are already logged into Anaconda Cloud ({auth_domain}). Would you like to force a new login?",
            default=False,
        )
        if not force:
            raise typer.Exit()

    login(force=force, ssl_verify=ssl_verify)


@app.command(name="whoami")
def auth_info() -> None:
    """Display information about the currently signed-in user"""
    api_config = APIConfig()

    if not (api_config.key or is_logged_in()):
        login()

    client = BaseClient()
    response = client.get("/api/account")
    console.print("Your Anaconda Cloud info:")
    console.print_json(data=response.json(), indent=2, sort_keys=True)


@app.command(name="api-key")
def auth_key() -> None:
    """Display API Key for signed-in user"""
    api_config = APIConfig()
    if api_config.key:
        console.print(api_config.key)
        return

    auth_config = AuthConfig()
    try:
        token_info = TokenInfo.load(domain=auth_config.domain)
        if not token_info.expired:
            console.print(token_info.api_key)
            return
    except TokenNotFoundError:
        pass

    login()
    token_info = TokenInfo.load(domain=auth_config.domain)
    console.print(token_info.api_key)


@app.command(name="logout")
def auth_logout() -> None:
    """Logout"""
    logout()



import pytest
from pytest import MonkeyPatch
from pytest_mock import MockerFixture
from typer.testing import CliRunner

from anaconda_cloud_auth.cli import app


@pytest.mark.usefixtures("disable_dot_env")
def test_api_key_not_logged_in(monkeypatch: MonkeyPatch, mocker: MockerFixture) -> None:
    monkeypatch.delenv("ANACONDA_CLOUD_API_KEY", raising=False)

    login = mocker.patch("anaconda_cloud_auth.cli.login")

    runner = CliRunner()
    _ = runner.invoke(app, ["api-key"])

    login.assert_called_once()


@pytest.mark.usefixtures("disable_dot_env")
def test_api_key_prefers_env_var(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setenv("ANACONDA_CLOUD_API_KEY", "foo")

    runner = CliRunner()
    result = runner.invoke(app, ["api-key"])
    assert result.exit_code == 0
    assert result.stdout.strip() == "foo"
